from car_framework.car_service import CarDbStatus
from car_framework.base_import import BaseImport
from car_framework.util import RecoverableFailure, check_for_error
from car_framework.context import context


class BaseFullImport(BaseImport):
    def __init__(self):
        super().__init__()


    def import_vertices(self):
        raise NotImplementedError()


    def import_edges(self):
        raise NotImplementedError()


    def create_source_report_object(self):
        raise NotImplementedError()


    def get_new_model_state_id(self):
        raise NotImplementedError()


    def init(self):
        db_status = context().car_service.get_db_status()
        if db_status == CarDbStatus.FAILURE:
            raise RecoverableFailure('Database is not ready.')

        source_report_data = self.create_source_report_object()
        status = context().car_service.import_data(source_report_data)
        check_for_error(status)

        self.statuses.append(status)
        self.wait_for_completion_of_import_jobs()
        context().car_service.enter_full_import_in_progress_state()
        self.new_model_state_id = self.get_new_model_state_id()


    def complete(self):
        context().car_service.exit_full_import_in_progress_state()
        self.save_new_model_state_id(self.new_model_state_id)
        context().logger.info('Done.')


    def run(self):
        self.init()
        self.import_vertices()
        self.wait_for_completion_of_import_jobs()
        self.import_edges()
        self.wait_for_completion_of_import_jobs()
        self.complete()



from enum import Enum


BATCH_SIZE = 20


def recoverable_failure_status_code(status_code):
    return status_code in (302, 400, 401, 403, 408, 500, 503, 504)


def check_for_error(status):
    if status.status != ImportJobStatus.FAILURE: return
    if recoverable_failure_status_code(status.status_code):
        raise RecoverableFailure('Import job failure. Status code: %d, Error: %s' % (status.status_code, status.error))
    else:
        raise UnrecoverableFailure('Import job failure. Status code: %d, Error: %s' % (status.status_code, status.error))


class RecoverableFailure(Exception):
    def __init__(self, message):
        from car_framework.context import context
        context().logger.error(message)
        self.message = message


class UnrecoverableFailure(Exception):
    def __init__(self, message):
        from car_framework.context import context
        context().logger.error(message)
        self.message = message


class IncrementalImportNotPossible(Exception):
    callback = None
    def __init__(self, message):
        from car_framework.context import context
        context().logger.info(message)
        self.message = message


def get_json(response):
    try: return response.json()
    except: return {}


class ImportJobStatus(object):

    FAILURE = 0
    IN_PROGRESS = 1
    SUCCESS = 2

    def __init__(self):
        self.status = ImportJobStatus.FAILURE
        self.status_code = 0
        self.error = 'Inknown'
